﻿using System;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Behaviors.Fleet;
using Amplitude.AI.Core.Brain.Tasks.FleetMissionTasks;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;

namespace Amplitude.AI.Core.Brain.FleetMissions
{
    public class FleetMission_Explore : FleetMission
    {
        public override void Initialize(AIPlayer aiPlayer, Task fleetMissionTask)
        {
            base.Initialize(aiPlayer, fleetMissionTask);
            this.snapshotRepositoryService = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
        }

        protected override void CheckMissionStatus()
        {
            FindAllNodesInConstellation findAllNodesInConstellation = base.Task as FindAllNodesInConstellation;
            ConstellationSnapshot constellationSnapshot;
            if (findAllNodesInConstellation != null && this.snapshotRepositoryService.GalaxySnapshot.TryGetConstellation(findAllNodesInConstellation.ContextGuid, out constellationSnapshot))
            {
                ConstellationData analysisData = constellationSnapshot.GetAnalysisData<ConstellationData>(base.EmpireSnapshot.EmpireIndex);
                if (analysisData.ExploredNodes >= analysisData.ReachableNodes)
                {
                    findAllNodesInConstellation.ChangeStatus(BlackboardMessage_Evaluable.EvaluationStatus.Succeed);
                }
            }
        }

        protected override void RegisterFleet(FleetSnapshot fleet)
        {
            BehaviorData behaviorData = new BehaviorData();
            behaviorData.FleetSnapshot = fleet;
            behaviorData.Status = BehaviorData.FleetBehaviorStatus.Ongoing;
            FindAllNodesInConstellation findAllNodesInConstellation = base.Task as FindAllNodesInConstellation;
            if (findAllNodesInConstellation != null && findAllNodesInConstellation.ContextGuid != 0UL)
            {
                behaviorData.MainConstellation = findAllNodesInConstellation.ContextGuid;
                behaviorData.BehaviorTree = new ExploreConstellation();
            }
            else
            {
                behaviorData.BehaviorTree = new ExploreClosestOne();
            }
            behaviorData.BehaviorTree.Initialize();
            base.BehaviorDataByFleetGuid.Add(fleet.GameEntityGuid, behaviorData);
        }

        public FleetMission_Explore()
        {
        }

        private ISnapshotRepositoryService snapshotRepositoryService;
    }
}
