﻿using System;
using Amplitude.AI.Core.Brain.Allocators;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Brain.Tasks.FleetMissionTasks;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.Position;

namespace Amplitude.AI.Core.Brain.Generators.Exploration
{
    public class SearchAllColonies : DefaultContextBasedGenerator<NodePosition, Win, SearchNodeCuriosities>
    {
        public override string LocalizedName
        {
            get
            {
                return "Search all the curiosities on a node.";
            }
        }

        public override string Description
        {
            get
            {
                return "Search all the curiosities on a node.";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 5;
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            MajorEmpireSnapshot majorEmpireSnapshot = aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            for (int i = 0; i < majorEmpireSnapshot.ColonizedStarSystems.Count; i++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot = majorEmpireSnapshot.ColonizedStarSystems[i];
                if (colonizedStarSystemSnapshot.IsActive && colonizedStarSystemSnapshot.GetAnalysisData<NodeData>(aiPlayer).CuriosityToSearch.Count != 0)
                {
                    base.ContinueDecision(aiPlayer, colonizedStarSystemSnapshot.NodePosition);
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, NodePosition context)
        {
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<Win>(VictorySnapshot.VictoryType.Score));
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, NodePosition context, Win parent)
        {
            SearchNodeCuriosities searchNodeCuriosities = aiPlayer.Blackboard.FindFirstRunning<SearchNodeCuriosities>(context);
            if (searchNodeCuriosities == null)
            {
                searchNodeCuriosities = new SearchNodeCuriosities(aiPlayer.Blackboard.GenerateGUID(), context);
                TaskWithInstance taskWithInstance = searchNodeCuriosities;
                ResourceAllocation[] array = new InstanceAllocation_Fleet[1];
                ResourceAllocation[] instanceAllocations = array;
                taskWithInstance.InstanceAllocations = instanceAllocations;
                searchNodeCuriosities.InstanceAllocations[0] = new InstanceAllocation_Fleet(ShipPattern.ShipRole.Exploration, 1);
                aiPlayer.Blackboard.Register<SearchNodeCuriosities>(searchNodeCuriosities);
            }
            base.ContinueDecision(aiPlayer, base.GetGenerationData(context, parent, searchNodeCuriosities));
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<NodePosition, Win, SearchNodeCuriosities> generationData)
        {
            return new HeuristicFloat(0.2f, "");
        }

        public SearchAllColonies()
        {
        }
    }
}
